@@ -64,46 +64,34 @@ private:
 class JavaInstance : public Instance
 {
 public:
-    JavaInstance (jobject instance);
+    JavaInstance (jobject instance, const RootObject *r);
         
     ~JavaInstance ();
     
     virtual Class *getClass() const;
     
-    JavaInstance (const JavaInstance &other);
-
-    JavaInstance &operator=(const JavaInstance &other){
-        if (this == &other)
-            return *this;
-        
-        JObjectWrapper *_oldInstance = _instance;
-        _instance = other._instance;
-        _instance->ref();
-        _oldInstance->deref();
-		
-        // Classes are kept around forever.
-        _class = other._class;
-        
-        return *this;
-    };
-
     virtual void begin();
     virtual void end();
     
-    virtual KJS::Value valueOf() const;
-    virtual KJS::Value defaultValue (KJS::Type hint) const;
+    virtual Value valueOf() const;
+    virtual Value defaultValue (Type hint) const;
 
-    virtual KJS::Value invokeMethod (KJS::ExecState *exec, const MethodList &method, const KJS::List &args);
+    virtual Value invokeMethod (ExecState *exec, const MethodList &method, const List &args);
+    virtual Value invokeDefaultMethod (ExecState *exec, const List &args);
 
     jobject javaInstance() const { return _instance->_instance; }
     
-    KJS::Value stringValue() const;
-    KJS::Value numberValue() const;
-    KJS::Value booleanValue() const;
-    
+    Value stringValue() const;
+    Value numberValue() const;
+    Value booleanValue() const;
+        
 private:
+    JavaInstance ();                         // prevent default construction
+    JavaInstance (JavaInstance &);           // prevent copying
+    JavaInstance &operator=(JavaInstance &); // prevent copying
+    
     JObjectWrapper *_instance;
-	mutable JavaClass *_class;
+    mutable JavaClass *_class;
 };
 
 } // namespace Bindings
