@@ -59,6 +59,8 @@ jvalue JSObject::invoke (JSObjectCallContext *context)
 {
     jvalue result;
 
+    bzero ((void *)&result, sizeof(jvalue));
+    
     if (!isJavaScriptThread()) {        
         // Send the call context to the thread that is allowed to
         // call JavaScript.
@@ -68,71 +70,76 @@ jvalue JSObject::invoke (JSObjectCallContext *context)
     else {
         jlong nativeHandle = context->nativeHandle;
         if (nativeHandle == UndefinedHandle || nativeHandle == 0) {
-            bzero ((void *)&result, sizeof(jvalue));
             return result;
         }
 
-        switch (context->type){
-            case CreateNative: {
-                result.j = JSObject::createNative(nativeHandle);
-                break;
-            }
-        
-            case Call: {
-                result.l = JSObject(nativeHandle).call(context->string, context->args);
-                break;
-            }
-            
-            case Eval: {
-                result.l = JSObject(nativeHandle).eval(context->string);
-                break;
-            }
-        
-            case GetMember: {
-                result.l = JSObject(nativeHandle).getMember(context->string);
-                break;
-            }
-            
-            case SetMember: {
-                JSObject(nativeHandle).setMember(context->string, context->value);
-                break;
-            }
-            
-            case RemoveMember: {
-                JSObject(nativeHandle).removeMember(context->string);
-                break;
-            }
-        
-            case GetSlot: {
-                result.l = JSObject(nativeHandle).getSlot(context->index);
-                break;
-            }
-            
-            case SetSlot: {
-                JSObject(nativeHandle).setSlot(context->index, context->value);
-                break;
-            }
-        
-            case ToString: {
-                result.l = (jobject) JSObject(nativeHandle).toString();
-                break;
+        if (context->type == CreateNative) {
+            result.j = JSObject::createNative(nativeHandle);
+        }
+        else {
+            KJS::ObjectImp *imp = jlong_to_impptr(nativeHandle);
+            if (!rootForImp(imp)) {
+                fprintf (stderr, "%s:%d:  Attempt to access JavaScript from destroyed applet, type %d.\n", __FILE__, __LINE__, context->type);
+                return result;
             }
 
-            case Finalize: {
-                ObjectImp *imp = jlong_to_impptr(nativeHandle);
-                if (findReferenceDictionary(imp) == 0) {
-                    // We may have received a finalize method call from the VM 
-                    // AFTER removing our last reference to the Java instance.
-                    JS_LOG ("finalize called on instance we have already removed.\n");
+            switch (context->type){            
+                case Call: {
+                    result.l = JSObject(nativeHandle).call(context->string, context->args);
+                    break;
                 }
-                else {
-                    JSObject(nativeHandle).finalize();
+                
+                case Eval: {
+                    result.l = JSObject(nativeHandle).eval(context->string);
+                    break;
                 }
-                break;
-            }
             
-            default: {
-                fprintf (stderr, "%s:  invalid JavaScript call\n", __PRETTY_FUNCTION__);
+                case GetMember: {
+                    result.l = JSObject(nativeHandle).getMember(context->string);
+                    break;
+                }
+                
+                case SetMember: {
+                    JSObject(nativeHandle).setMember(context->string, context->value);
+                    break;
+                }
+                
+                case RemoveMember: {
+                    JSObject(nativeHandle).removeMember(context->string);
+                    break;
+                }
+            
+                case GetSlot: {
+                    result.l = JSObject(nativeHandle).getSlot(context->index);
+                    break;
+                }
+                
+                case SetSlot: {
+                    JSObject(nativeHandle).setSlot(context->index, context->value);
+                    break;
+                }
+            
+                case ToString: {
+                    result.l = (jobject) JSObject(nativeHandle).toString();
+                    break;
+                }
+    
+                case Finalize: {
+                    ObjectImp *imp = jlong_to_impptr(nativeHandle);
+                    if (findReferenceDictionary(imp) == 0) {
+                        // We may have received a finalize method call from the VM 
+                        // AFTER removing our last reference to the Java instance.
+                        JS_LOG ("finalize called on instance we have already removed.\n");
+                    }
+                    else {
+                        JSObject(nativeHandle).finalize();
+                    }
+                    break;
+                }
+                
+                default: {
+                    fprintf (stderr, "%s:  invalid JavaScript call\n", __PRETTY_FUNCTION__);
+                }
             }
         }
         context->result = result;
@@ -152,7 +159,7 @@ JSObject::JSObject(jlong nativeJSObject)
     
     _root = rootForImp(_imp);
     
-    // If we can't find the root for the object something is terrible wrong.
+    // If we can't find the root for the object something is terribly wrong.
     assert (_root != 0);
 }
 
@@ -163,11 +170,10 @@ jobject JSObject::call(jstring methodName, jobjectArray args) const
 
     // Lookup the function object.
     ExecState *exec = _root->interpreter()->globalExec();
-    Interpreter::lock();
+    InterpreterLock lock;
     
     Identifier identifier(JavaString(methodName).ustring());
     Value func = _imp->get (exec, identifier);
-    Interpreter::unlock();
     if (func.isNull() || func.type() == UndefinedType) {
         // Maybe throw an exception here?
         return 0;
@@ -177,22 +183,31 @@ jobject JSObject::call(jstring methodName, jobjectArray args) const
     ObjectImp *funcImp = static_cast<ObjectImp*>(func.imp());
     Object thisObj = Object(const_cast<ObjectImp*>(_imp));
     List argList = listFromJArray(args);
-    Interpreter::lock();
-    Value result = funcImp->call (exec, thisObj, argList);
-    Interpreter::unlock();
 
-    // Convert and return the result of the function call.
-    return convertValueToJObject (result);
+    Value result = Object(funcImp).call (exec, thisObj, argList);
+
+    return convertValueToJObject(result);
 }
 
 jobject JSObject::eval(jstring script) const
 {
     JS_LOG ("script = %s\n", JavaString(script).UTF8String());
-
+    
     Object thisObj = Object(const_cast<ObjectImp*>(_imp));
-    Interpreter::lock();
-    KJS::Value result = _root->interpreter()->evaluate(UString(), 0, JavaString(script).ustring(),thisObj).value();
-    Interpreter::unlock();
+    Value result;
+    
+    InterpreterLock lock;
+    
+    Completion completion = _root->interpreter()->evaluate(UString(), 0, JavaString(script).ustring(),thisObj);
+    ComplType type = completion.complType();
+    
+    if (type == Normal) {
+        result = completion.value();
+        if (result.isNull())
+            result = Undefined();
+    } else
+        result = Undefined();
+    
     return convertValueToJObject (result);
 }
 
@@ -202,20 +217,18 @@ jobject JSObject::getMember(jstring memberName) const
 
     ExecState *exec = _root->interpreter()->globalExec();
 
-    Interpreter::lock();
+    InterpreterLock lock;
     Value result = _imp->get (exec, Identifier (JavaString(memberName).ustring()));
-    Interpreter::unlock();
 
-    return convertValueToJObject (result);
+    return convertValueToJObject(result);
 }
 
 void JSObject::setMember(jstring memberName, jobject value) const
 {
     JS_LOG ("memberName = %s, value = %p\n", JavaString(memberName).UTF8String(), value);
     ExecState *exec = _root->interpreter()->globalExec();
-    Interpreter::lock();
-    _imp->put (exec, Identifier (JavaString(memberName).ustring()), convertJObjectToValue(value));
-    Interpreter::unlock();
+    InterpreterLock lock;
+    _imp->put(exec, Identifier (JavaString(memberName).ustring()), convertJObjectToValue(value));
 }
 
 
@@ -224,49 +237,44 @@ void JSObject::removeMember(jstring memberName) const
     JS_LOG ("memberName = %s\n", JavaString(memberName).UTF8String());
 
     ExecState *exec = _root->interpreter()->globalExec();
-    Interpreter::lock();
-    _imp->deleteProperty (exec, Identifier (JavaString(memberName).ustring()));
-    Interpreter::unlock();
+    InterpreterLock lock;
+    _imp->deleteProperty(exec, Identifier (JavaString(memberName).ustring()));
 }
 
 
 jobject JSObject::getSlot(jint index) const
 {
-    JS_LOG ("index = %d\n", index);
+    JS_LOG ("index = %ld\n", index);
 
     ExecState *exec = _root->interpreter()->globalExec();
-    Interpreter::lock();
-    Value result = _imp->get (exec, (unsigned)index);
-    Interpreter::unlock();
 
-    return convertValueToJObject (result);
+    InterpreterLock lock;
+    Value result = _imp->get (exec, (unsigned)index);
+
+    return convertValueToJObject(result);
 }
 
 
 void JSObject::setSlot(jint index, jobject value) const
 {
-    JS_LOG ("index = %d, value = %p\n", index, value);
+    JS_LOG ("index = %ld, value = %p\n", index, value);
 
     ExecState *exec = _root->interpreter()->globalExec();
-    Interpreter::lock();
-    _imp->put (exec, (unsigned)index, convertJObjectToValue(value));
-    Interpreter::unlock();
+    InterpreterLock lock;
+    _imp->put(exec, (unsigned)index, convertJObjectToValue(value));
 }
 
 
 jstring JSObject::toString() const
 {
     JS_LOG ("\n");
-
-    Interpreter::lock();
+    
+    InterpreterLock lock;
     Object thisObj = Object(const_cast<ObjectImp*>(_imp));
+
     ExecState *exec = _root->interpreter()->globalExec();
     
-    jstring result = (jstring)convertValueToJValue (exec, thisObj, object_type, "java.lang.String").l;
-
-    Interpreter::unlock();
-    
-    return result;
+    return (jstring)convertValueToJValue (exec, thisObj, object_type, "java.lang.String").l;
 }
 
 void JSObject::finalize() const
@@ -398,9 +406,7 @@ KJS::Value JSObject::convertJObjectToValue (jobject theObject) const
     // figure 22-4.
     jobject classOfInstance = callJNIObjectMethod(theObject, "getClass", "()Ljava/lang/Class;");
     jstring className = (jstring)callJNIObjectMethod(classOfInstance, "getName", "()Ljava/lang/String;");
-    
-    JS_LOG ("converting instance of class %s\n", Bindings::JavaString(className).UTF8String());
-    
+        
     if (strcmp(Bindings::JavaString(className).UTF8String(), "netscape.javascript.JSObject") == 0) {
         // Pull the nativeJSObject value from the Java instance.  This is a
         // pointer to the ObjectImp.
@@ -417,11 +423,8 @@ KJS::Value JSObject::convertJObjectToValue (jobject theObject) const
         return KJS::Object(const_cast<KJS::ObjectImp*>(imp));
     }
 
-    Interpreter::lock();
-    KJS::RuntimeObjectImp *newImp = new KJS::RuntimeObjectImp(new Bindings::JavaInstance (theObject));
-    Interpreter::unlock();
-
-    return KJS::Object(newImp);
+    InterpreterLock lock;
+    return KJS::Object(new RuntimeObjectImp(new Bindings::JavaInstance (theObject, _root)));
 }
 
 KJS::List JSObject::listFromJArray(jobjectArray jArray) const
@@ -432,8 +435,14 @@ KJS::List JSObject::listFromJArray(jobjectArray jArray) const
     
     for (i = 0; i < numObjects; i++) {
         jobject anObject = env->GetObjectArrayElement ((jobjectArray)jArray, i);
-        aList.append (convertJObjectToValue(anObject));
-        env->DeleteLocalRef (anObject);
+	if (anObject) {
+	    aList.append (convertJObjectToValue(anObject));
+	    env->DeleteLocalRef (anObject);
+	}
+	else {
+            env->ExceptionDescribe();
+            env->ExceptionClear();
+	}
     }
     return aList;
 }
