@@ -419,14 +419,22 @@ void _NPN_ReleaseObject (NPObject *obj)
     assert (obj->referenceCount >= 1);
 
     if (obj && obj->referenceCount >= 1) {
-        obj->referenceCount--;
-                
-        if (obj->referenceCount == 0) {
-            if (obj->_class->deallocate)
-                obj->_class->deallocate (obj);
-            else
-                free (obj);
-        }
+        obj->referenceCount--;                
+        
+        if (obj->referenceCount == 0)
+            _NPN_DeallocateObject(obj);
+    }
+}
+
+void _NPN_DeallocateObject(NPObject *obj)
+{
+    assert(obj);
+
+    if (obj) {
+        if (obj->_class->deallocate)
+            obj->_class->deallocate(obj);
+        else
+            free(obj);
     }
 }
 
