@@ -372,7 +372,7 @@ int main(int argc, char **argv)
     
     bool ret = true;
     {
-        Interpreter::lock();
+        InterpreterLock lock;
         
         // create interpreter w/ global object
         Object global(new GlobalImp());
@@ -414,8 +414,6 @@ int main(int argc, char **argv)
                 
         NPN_ReleaseObject ((NPObject *)myObject);
         
-        Interpreter::unlock();
-        
     } // end block, so that Interpreter and global get deleted
     
     return ret ? 0 : 3;
