@@ -180,6 +180,18 @@ namespace KJS {
     bool hasProperty(ExecState *exec, const Identifier &propertyName) const;
     bool hasProperty(ExecState *exec, unsigned propertyName) const;
 
+    /**
+     * Checks to see whether the object has a property with the specified name.
+     *
+     * See ECMA 15.2.4.5
+     *
+     * @param exec The current execution state
+     * @param propertyName The name of the property to check for
+     * @return true if the object has the property, otherwise false
+     */
+    bool hasOwnProperty(ExecState *exec, const Identifier &propertyName) const;
+    bool hasOwnProperty(ExecState *exec, unsigned propertyName) const;
+
     /**
      * Removes the specified property from the object.
      *
@@ -244,6 +256,7 @@ namespace KJS {
      * @return The newly created &amp; initialized object
      */
     Object construct(ExecState *exec, const List &args);
+    Object construct(ExecState *exec, const List &args, const UString &sourceURL, int lineNumber);
 
     /**
      * Whether or not the object implements the call() method. If this returns
@@ -511,9 +524,13 @@ namespace KJS {
      *
      * @see Object::hasProperty()
      */
-    virtual bool hasProperty(ExecState *exec,
+    bool hasProperty(ExecState *exec,
 			     const Identifier &propertyName) const;
-    virtual bool hasProperty(ExecState *exec, unsigned propertyName) const;
+    bool hasProperty(ExecState *exec, unsigned propertyName) const;
+
+    virtual bool hasOwnProperty(ExecState *exec,
+    		     const Identifier &propertyName) const;
+    virtual bool hasOwnProperty(ExecState *exec, unsigned propertyName) const;
 
     /**
      * Implementation of the [[Delete]] internal property (implemented by all
@@ -547,6 +564,7 @@ namespace KJS {
      * @see Object::construct()
      */
     virtual Object construct(ExecState *exec, const List &args);
+    virtual Object construct(ExecState *exec, const List &args, const UString &sourceURL, int lineNumber);
 
     virtual bool implementsCall() const;
     /**
@@ -635,7 +653,7 @@ namespace KJS {
      */
     static Object create(ExecState *exec, ErrorType errtype = GeneralError,
                          const char *message = 0, int lineno = -1,
-                         int sourceId = -1);
+                         int sourceId = -1, const UString *sourceURL = 0);
 
     /**
      * Array of error names corresponding to @ref ErrorType
@@ -680,6 +698,12 @@ namespace KJS {
   inline bool Object::hasProperty(ExecState *exec, unsigned propertyName) const
     { return imp()->hasProperty(exec, propertyName); }
 
+  inline bool Object::hasOwnProperty(ExecState *exec, const Identifier &propertyName) const
+    { return imp()->hasOwnProperty(exec, propertyName); }
+
+  inline bool Object::hasOwnProperty(ExecState *exec, unsigned propertyName) const
+    { return imp()->hasOwnProperty(exec, propertyName); }
+
   inline bool Object::deleteProperty(ExecState *exec, const Identifier &propertyName)
     { return imp()->deleteProperty(exec,propertyName); }
 
@@ -694,6 +718,9 @@ namespace KJS {
 
   inline Object Object::construct(ExecState *exec, const List &args)
     { return imp()->construct(exec,args); }
+  
+  inline Object Object::construct(ExecState *exec, const List &args, const UString &sourceURL, int lineNumber)
+  { return imp()->construct(exec,args,sourceURL,lineNumber); }
 
   inline bool Object::implementsCall() const
     { return imp()->implementsCall(); }
